##Cristina Cabrera
##Game Engine Project
##December 8, 2010

class State:
    def __init__(self, name, description):
        self.name = name
        self.description = description
        self.edges = []
        self.constraints = []
        self.objects = []


class Edge:
    def __init__(self, name, from_node, commands, to_node, details, health, constraints=None):
        self.name = name
        self.from_node = from_node 
        self.commands = commands
        self.to_node = to_node
        self.details = details
        self.health = health
        self.constraints = constraints
        self.contains_constraint = False
        

class Constraint:
    def __init__(self, name, object_name, state_name):
        self.name = name
        self.object_name = object_name
        self.state_name = state_name
        self.completed = False
        
class Object:
    def __init__(self, location_name, unique_name, short_description, long_description, weight):
        self.location_name = location_name
        self.unique_name = unique_name
        self.short_description = short_description
        self.long_description = long_description
        self.weight = weight
    

class Backpack:
    def __init__(self):
        self.objects = []
        self.weight = 0

class Audio:
    def __init__(self, audio_file):
        self.audio_file = audio_file

class Help:
    def __init__(self, name, tip):
        self.name = name
        self.tip = tip

class Directions:
    def __init__(self, directions):
        self.directions = directions
        
def word_wrap(text, col=70):
    text_wrap = ""
    count = 0
    for character in text:
        if count > col and character == " ":
            text_wrap = text_wrap + "\n"
            count = 0
        else:
            text_wrap = text_wrap + character
            count = count + 1
    return text_wrap

import time
import random
import pygame.mixer                                           
import os

def background_music(audio):
    pygame.mixer.init()                                              
    pygame.mixer.music.load(audio)    
    pygame.mixer.music.play(-1)

    
class Engine:
    def __init__(self, filename):
        self.filename = filename
        self.initial_state = None
        self.health = 20
        self.counter = 0
        self.help = None
        self.constraints = []
        self.backpack = Backpack()
        self.bag = False
        self.audio = None
        self.directions = None

    def load(self):
        self.states = {}
        edges = []
        objects = []
        constraints = []
        fp = open(self.filename)
        for line in fp:
            if len(line) == 0 or line[0] == '#':
                continue
            items = line.strip().split("::")
            if items[0] == 'node':
                s = State(*items[1:])
                if self.initial_state is None:
                    self.initial_state = s
                self.states[items[1]] = s
            elif items[0] == 'edge':
                edges.append(Edge(*items[1:]))
            elif items[0] == 'object':
                objects.append(Object(*items[1:]))
            elif items[0] == 'backpack':
                self.bag is True
                self.max_weight = int(items[2])
            elif items[0] == 'help':
                tips = Help(*items[1:])
                if self.help is None:
                    self.help = tips
            elif items[0] == 'audio':
                audio = Audio(*items[1:])
                if self.audio is None:
                    self.audio = audio
            elif items[0] == 'directions':
                directions = Directions(*items[1:])
                if self.directions is None:
                    self.directions = directions
            elif items[0] == 'constraint':
                self.constraints.append(Constraint(*items[1:]))   
        for edge in edges:
            self.states[edge.from_node].edges.append(edge)
        for obj in objects:
            self.states[obj.location_name].objects.append(obj)
            
    def check(self, constraint):
        if constraint.object_name == 'eval':
            if eval(constraint.state_name) is True:
                constraint.completed = True
                return constraint.completed
        elif constraint.object_name == 'exec':
            contraint.completed = True
            exec(constraint.state_name)
        else:
            if constraint.state_name == 'backpack':
                items = constraint.object_name.split(' , ')
                for item in self.backpack.objects:
                    if item.unique_name in items:
                        constraint.completed = True
                        return constraint.completed
            elif constraint.state_name in self.states:
                for item in self.states[constraint.state_name].objects:
                    if item in self.states[constraint.state_name].objects:
                        constraint.completed = True
                        return constraint.completed
            
        
    def play(self):
        t = time.time()
        current_state = self.initial_state
        objects = []
        name = input("Please Enter Your Name> ")
        print('Welcome ',name,'! Good Luck and Enjoy the Game!')
        if self.audio is not None:
            background_music(self.audio.audio_file)
        while True:
            print(current_state.description)
            command = input("Command> ")
            for edge in current_state.edges:
                if command in edge.commands:
                    if edge.constraints is not None:
                        edge.contains_constraint = True
                        for constraint in self.constraints:
                            if constraint.name in edge.constraints:
                                self.check(constraint)
                                
                                if constraint.completed is True:
                                    edge.contains_constraint = False
                                    current_state = self.states[edge.to_node]
                                    self.counter = self.counter + 1
                                    self.health = self.health + int(edge.health)
                                    print(word_wrap(edge.details))
                                    print(int(time.time() - t))
                                    
                                elif constraint.completed is False:
                                    edge.contains_constraint = True
                                    print("Sorry, ", name, " you can't make that move yet. You might be missing something...")
                                    self.counter = self.counter + 1
                                    print(int(time.time() - t))
                                    
                    else:      
                        current_state = self.states[edge.to_node]
                        print(word_wrap(edge.details))
                        self.counter = self.counter + 1
                        self.health = self.health + int(edge.health)
                        print(int(time.time() - t))
                        
                if self.health <= 0:
                    print("Sorry to say this ", name, " but you health level has reached 0. GAME OVER!")
                    response = input(name, ", would you like to play again?(yes or no)")
                    if response == "yes":
                        self.health = 20
                        x.play()
                    else:
                        print("Thanks for playing ", name)
                        pygame.mixer.quit()
                        break
                    
            x = command.split()        
            for obj in current_state.objects:
                if x[0] == 'get':
                    if x[1] in obj.unique_name:
                        if (self.backpack.weight + int(obj.weight)) < self.max_weight:
                            self.backpack.objects.append(obj)
                            current_state.objects.remove(obj)
                            self.backpack.weight = self.backpack.weight + int(obj.weight)
                            print("You picked up the ", obj.unique_name)
                            print(int(time.time() - t))
                        else:
                            print("Sorry,", name, " you cannot carry anymore items.")
            if x[0] == 'drop':
                if x[1] in obj.unique_name:
                    self.backpack.objects.remove(obj)
                    print("You just dropped ", obj.unique_name)
                    print(int(time.time() - t))
            if x[0] == 'eat':
                if x[1] in obj.unique_name:
                    self.backpack.objects.remove(obj)
                    print("You have eaten the ", obj.unique_name)
                    print(int(time.time() - t))
            if x[0] == 'use':
                if x[1] in obj.unique_name:
                    self.backpack.objects.remove(obj)
                    print("You have used the ", obj.unique_name)
                    print(int(time.time() - t))
                        
            if 'backpack' in command:
                print("Your backpack contains ")
                for item in range(len(self.backpack.objects)):
                    print(self.backpack.objects[item].unique_name)
                    print("backpack weight: ", self.backpack.weight)
            if 'help' in command:
                if self.help is not None:
                    if current_state.name in self.help.name:
                        print ("Here's your Helpful tip ", name,": ", self.help.tip)
            if 'health' in command:
                print(name, "'s Health Status: ", self.health)
            if 'directions' in command:
                if self.directions is not None:
                    print("This is how you play the game", name, "!", self.directions.directions)
            if 'quit' in command:
                print('Thanks for playing', name , '! Bye!')
                pygame.mixer.quit()
                break
            

                
                
            
               
x = Engine('princess.txt')
x.load()
x.play()


